var bit = bit || {};

if (typeof window.Blob !== 'function') {
	/**
	 * Wrap a Blob object inside a function to turn it into a constructor, using the *deprecated* BlobBuilder object.
	 * @param {Array} blobParts 	An Array of data objects to put into the new Blob object. This can be any number of ArrayBuffer, ArrayBufferView (typed array), Blob, or DOMString objects, in any order.
	 * @param {Object} options		An object that provides the properties for the new Blob. Used to specify type, eg 'text\/html' or 'application/octet-binary'
	 * @see http://www.w3.org/TR/FileAPI/#dfn-Blob
	 * @see https://developer.mozilla.org/en-US/docs/DOM/Blob
	 */
	window.Blob = function(blobParts, options) {
		var type = options ? (options.type || '') : '';
		var builder = new window.WebKitBlobBuilder();
		if (blobParts) {
			for (var i = 0; i < blobParts.length; i++) {
				builder.append(blobParts[i]);
			}
		}
		// returns an actual Blob object, so now we can use 'new Blob()'
		return builder.getBlob(type);
	};
}

function FileIO()
{
	this.IO = window.nwf ? window.nwf.io : false;
	this.allFiles;
	this.currentFile = -1;
	this.locked = false;
	this.uniqueId = 0;
	this.fileReader = new FileReader();
	
	if (this.IO) {
	this.writeDir = this.IO.Directory.appCommonSaveDirectory.systemPath;
	this.writeDirObj = new this.IO.Directory(this.writeDir);
	}
	
	this.canWrite = true;
	
	this.DATA = {};
	
	
	//Save File
	this.saveFile = function(filename, data, readyFunc, failFunc)
	{
		if (!this.canWrite)
		{
			if (failFunc) failFunc();
			return false;
		}
		
		if (!this.IO) {
			this.DATA[filename] = data;
			if (readyFunc) readyFunc();
			return true;
		}
		
		data = JSON.stringify(data);
		
		var readOnly = this.writeDirObj.readonly;
		var file = new this.IO.File(filename, this.writeDirObj);
		
		var blob = new Blob( [data], {type: 'text/html'} );
		file.addEventListener(nwf.events.IOEvent.SAVE_COMPLETE, onSaveComplete, this);
		file.addEventListener(nwf.events.IOEvent.ERROR, onFail, this);
		file.save(blob);
		
		function onSaveComplete(evt)
		{
			evt.target.removeEventListener(nwf.events.IOEvent.SAVE_COMPLETE, onSaveComplete, this);
			evt.target.removeEventListener(nwf.events.IOEvent.ERROR, onFail, this);
			
			if (readyFunc) readyFunc();
		};
		function onFail(evt)
		{
			evt.target.removeEventListener(nwf.events.IOEvent.SAVE_COMPLETE, onSaveComplete, this);
			evt.target.removeEventListener(nwf.events.IOEvent.ERROR, onFail, this);

			if (failFunc) failFunc();
		};
	};
	
	
	//Delete this file
	this.deleteFile = function(filename)
	{
		if (!this.IO) {
			delete this.DATA[filename];
			return true;
		}
		
		this.IO.File.removeFile(this.writeDir + filename);
	};
	
	
	//Read File
	this.readFile = function(filename, readyFunc, failFunc)
	{
		if (!this.IO) {
			var data = this.DATA[filename];
			if (data != undefined) readyFunc(data);
			else failFunc();
			return true;
		}
		
		//JSON.parse
		var file = new this.IO.File(filename, this.writeDirObj);
		file.addEventListener(nwf.events.IOEvent.READ_COMPLETE, onReadComplete, this);
		file.addEventListener(nwf.events.IOEvent.ERROR, onFail, this);
		file.read();
		
		function onReadComplete(evt)
		{
			evt.target.removeEventListener(nwf.events.IOEvent.READ_COMPLETE, onReadComplete, this);
			evt.target.removeEventListener(nwf.events.IOEvent.ERROR, onFail, this);
			
			FileIO.fileReader.onloadend = function(evt) {
				if (FileIO.fileReader.error) {
					
					return;
				}
				
				readyFunc(JSON.parse(evt.target.result));
			};
			FileIO.fileReader.readAsText(evt.data);
		};
		function onFail(evt)
		{
			evt.target.removeEventListener(nwf.events.IOEvent.READ_COMPLETE, onReadComplete, this);
			evt.target.removeEventListener(nwf.events.IOEvent.ERROR, onFail, this);

			if (failFunc) failFunc();
		};
	};
	
	
	//Does this file exist?
	this.fileExists = function(filename)
	{
		if (!this.IO) {
			return this.DATA[filename] != undefined;
		}
		
		//$("#debug").append("1-0<br/>");
		try
		{
			//$("#debug").append("1-1<br/>");
			var file = new this.IO.File(filename, this.writeDirObj);
			//$("#debug").append("1-2<br/>");
			return file.exists;
		} catch(err) {
			//$("#debug").append("1-3<br/>");
			return false;
		}
	};
};
var FileIO = new FileIO();